ns = function(name){
    var array = [];
    if (name.indexOf('.') !== -1){
        array = name.split('.');
    }else{
        array.push(name);
    }
    var scope = this;
    $.each(array, function(index, namespace){
        if (scope[namespace] == null || scope[namespace] == undefined){
            scope[namespace] = {};
        }
        scope = scope[namespace];
    });
}

extend = function(f, parents, config){
    if (parents){
        if ($.isArray(parents)){
            $.each(parents, function(index, parent){
                $.extend(f.prototype, parent.prototype);
            });
        }else if ($.isFunction(parents)){
            $.extend(f.prototype, parents.prototype);
        }else {
            $.extend(f.prototype, parents);
        }
    }
    if (config){
        $.extend(f.prototype, config);
    }
}


ns('wapi');

wapi.url = '/wapi/v1.0/';

ns('wapi.event');

/* singleton object - system bus for event firing and listening */
wapi.event.MessageBus = {

    subscribe: function(event, fn) {
        $(this).bind(event, fn);
    },

    unsubscribe: function(event, fn) {
        $(this).unbind(event, fn);
    },

    fireEvent: function(event, params) {
        $(this).trigger(event, params);
    }
}

/* component with possibility to fire/listen any event using global system bus - muzlo.event.MessageBus */
wapi.event.EventProxy = function(){};

wapi.event.EventProxy.prototype = {

    _listeners: {},

    publish: function(event, params){
        wapi.event.MessageBus.fireEvent(event, params);
    },

    on: function(event, listener){
        var f = listener.bind(this);
        this._listeners[event] = f;
        wapi.event.MessageBus.subscribe(event, f);
    },

    un: function(event, listener){
        var f = this._listeners[event];
        wapi.event.MessageBus.unsubscribe(event, f);
        this._listeners[event] = null;
    }

}

ns('wapi.widget');

wapi.widget.Grid = function(config){
	
	for (var prop in config){
		this[prop] = config[prop];
		
	}
	
	var el = $('#' + this.selector);
	
	el.append('<div id="' + this.selector + '-header' + '"></div>');
	el.append('<div id="' + this.selector + '-rows' + '"></div>')
	
	
	this.headerContainer = $('#' + this.selector + '-header');
	this.rowContainer = $('#' + this.selector + '-rows');
	
	this.initEventListeners();
	
	this.start();
	
	
};

extend(wapi.widget.Grid, wapi.event.EventProxy, {

    objectType: '',
	
	el: '',
	
	// object like : {title: '', dataIndex: '', renderer: function(){}}
	columns: [],
	
	createHeaders: function(headerContainer, columns){
		headerContainer.empty();
		var content = '<table><tr>';
		$.each(columns, function(index, column){
			content = content + '<td class="grid-header">' + column.title + '</td>';
		});	
		content = content + '<td class="grid-header">Actions</td>';
		headerContainer.append(content + '</td></table>');
	},
	
    createRows: function(container, items, columns){
        
		container.off('click', 'tr', this.onRowClick);
		container.empty();
		
		var content = '<table>';
		$.each(items, function(index, item){
			content = content + '<tr>';
			$.each(columns, function(index, column){
				content = content + '<td>';
				if (column.renderer) {
					content = content + column.renderer(item[column.dataIndex], item);
				} else {
					content =  content + item[column.dataIndex];
				}
				content = content + '</td>';
			});	
			content = content + '<td><input type="button" value="Delete"/></td>';
			content = content + '</tr>';
        });
		container.append(content + '</table>');
		
		container.on('click', 'tr', this, this.onRowClick);
    },
	
	onDeleteException: function(response, textStatus, errorThrown){
		alert(JSON.parse(response.responseText).text);
	},
	
    onRowClick: function(event) {
		var scope = event.data;
		
		var row = $(this);
		var selected = scope.data[row.index()];
		
		if (event.target.localName === 'input'){
			$.ajax(wapi.url + selected._ref, {
				type: 'DELETE',
				success: scope.start.bind(scope),
				error: scope.onDeleteException.bind(scope)
			});	
		} else {
			$(event.delegateTarget).find('tr.ui-selected').removeClass('ui-selected');
			row.addClass('ui-selected');
			wapi.event.MessageBus.fireEvent('rowselect', {
				objectType: scope.objectType,
				selected: selected
			});
		}
		
		
	},
	
	
    onLoad: function(data, textStatus){
		this.data = data;
		this.createHeaders(this.headerContainer, this.columns);
		this.createRows(this.rowContainer, data, this.columns);
    },
	
	start: function(){
		$.ajax(wapi.url + this.objectType, {
			success: this.onLoad.bind(this)	
		});
	},
	
	initEventListeners: function(){
		wapi.event.MessageBus.subscribe('objectupdated', this.start.bind(this));	
	}

});

wapi.widget.Editor = function(config){
	
	for (var prop in config){
		this[prop] = config[prop];
		
	}
	
	this.initEventListeners();
}
extend(wapi.widget.Editor, {
	
	objectType: '',
	
	saveButtonSelector: '',

	clearButtonSelector: '',	
	
	deleteButtonSelector: '',
	
	initEventListeners: function(){
		wapi.event.MessageBus.subscribe('rowselect', function(event, data){
			if (this.objectType == data.objectType){
				this.fillEditor(data.selected);
				
				$('#' + this.saveButtonSelector).val('Update');
				
			}
		}.bind(this));
		
		$('#' + this.clearButtonSelector).click(function(){
			this.clearEditor();
			$('#' + this.saveButtonSelector).val('Add');
		}.bind(this));
		
		$('#' + this.saveButtonSelector).click(function(){
			var obj = this.getObject();
			if (obj._ref == null || obj._ref == ''){
				delete obj._ref;	
				this.create(obj);
			} else {
				this.update(obj);
			}
		}.bind(this));
		
		$('#' + this.deleteButtonSelector).click(function(){
			this.deleteObject(this.getObject());
		}.bind(this));		
	},
	
	deleteObject: function(obj){
		if (obj._ref && obj._ref != ''){
			$.ajax(wapi.url + obj._ref, {
				type: 'DELETE',
				success: this.onSuccessOperation.bind(this),
				error: this.onFailedOperation.bind(this)
			});	
		}
	},
	
	create: function(obj){
		$.ajax(wapi.url + this.objectType, {
			type: 'POST',
			data: obj,
			success: this.onSuccessOperation.bind(this),
			error: this.onFailedOperation.bind(this)
		});
	},
	
	update: function(obj){
		var ref = obj._ref;
		delete obj._ref;
		$.ajax(wapi.url + ref, {
			type: 'PUT',
			data: obj,
			success: this.onSuccessOperation.bind(this),
			error: this.onFailedOperation.bind(this)
		});	
	},
	
	onFailedOperation: function(response, textStatus, errorThrown){
		this.clearEditor();
		$('#' + this.saveButtonSelector).val('Add');
		alert(JSON.parse(response.responseText).text);
	},
	
	onSuccessOperation: function(data, textStatus, jqXHR){
		this.clearEditor();
		$('#' + this.saveButtonSelector).val('Add');
		wapi.event.MessageBus.fireEvent('objectupdated', this.objectType);
	},
	
	fillEditor: function(selected){

	},

    clearEditor: function(){
		
	},

	getObject: function(){
		
	}
});

